var _ = require('highland');
var Table = require('cli-table');

function formatDate(date) {
  var year = date.getFullYear().toString();
  var month = (date.getMonth() + 1).toString();
  var mm = month[1] ? month : '0' + month;
  var day = date.getDate().toString();
  var dd = day[1] ? day : '0' + day;
  return year + '-' + mm + '-' + dd;
}

function sortAscending(a, b) {
  return a - b;
}

function getToday() {
  return new Date();
}

function getMonthAgo() {
  return new Date((new Date()).setMonth((new Date()).getMonth() - 1));
}

function sortDates(date1, date2) {
  var utc1 = new Date(date1);
  var utc2 = new Date(date2);
  return utc2 - utc1;
}

function sum(arr) {
  var len = arr.length;
  var total = 0;
  for (var i = 0; i < len; ++i) {
    total += arr[i];
  }
  return total;
}

function getStats(arr) {
  arr.sort(sortAscending);
  var total = sum(arr);
  var obj = {
    max: arr[arr.length - 1],
    avg: (total / arr.length).toFixed(0),
    med: arr[Math.round(arr.length * 0.5)],
    25: arr[Math.round(arr.length * 0.25)],
    75: arr[Math.round(arr.length * 0.75)]
  };
  return obj;
}

function transformStreamJQL(str) {
  return _(function(push) {
    var data;
    str.on('data', function(chunk) {
      data = chunk;
    });
    str.on('end', function() {
      var dates = Object.keys(data);
      dates.sort(sortDates);
      var len = dates.length;
      for (var i = 0; i < len; ++i) {
        push(null, {
          date: dates[i],
          values: data[dates[i]]
        });
      }
      push(null, _.nil);
    });
  });
}

function transformStreamFunnel(str) {
  return _(function(push) {
    str.on('data', function(chunk) {
      var len = chunk.meta.dates.length;
      var date;
      for (var i = len - 1; i >= 0; --i) {
        date = chunk.meta.dates[i];
        var obj = {};
        obj.date = date;
        obj.values = {};
        obj.values.analysis = {
          starting_amount: chunk.data[date].analysis.starting_amount,
          completion: chunk.data[date].analysis.completion
        };
        obj.values.steps = [];
        var stepLen = chunk.data[date].steps.length;
        for (var j = 0; j < stepLen; ++j) {
          obj.values.steps.push({
            event: chunk.data[date].steps[j].event,
            count: chunk.data[date].steps[j].count
          });
        }
        push(null, obj);
      }
      push(null, _.nil);
    });
  });
}

function makeTable(keys, arr) {
  var table = new Table({
    head: keys,
    style: {
      head: ['green']
    }
  });
  for (var i = 0; i < arr.length; ++i) {
    var newRow = [];
    for (var j = 0; j < keys.length; ++j) {
      newRow.push(arr[i][keys[j]] || 0);
    }
    table.push(newRow);
  }
  return table;
}

function isInArray(value, array) {
  return array.indexOf(value) > -1;
}

module.exports = {
  formatDate: formatDate,
  getToday: getToday,
  getMonthAgo: getMonthAgo,
  sortAscending: sortAscending,
  sortDates: sortDates,
  transformStreamJQL: transformStreamJQL,
  transformStreamFunnel: transformStreamFunnel,
  sum: sum,
  getStats: getStats,
  monthInterval: 32,
  makeTable: makeTable,
  isInArray: isInArray
};
