/* eslint-env es6 */

var mixpanel = require('mixpanel-api-wrapper');
var helpers = require('./helpers.js');

var today = helpers.getToday();

var monthAgo = helpers.getMonthAgo();

var sessionStatsQuery = `function main() {
  return Events({
    from_date: '${helpers.formatDate(monthAgo)}',
    to_date: '${helpers.formatDate(today)}'
  })
  .filter(function(event) {
    return /^((?!(dev|world|testing)).)*$/.test(event.properties['app version'])
  })
  .filter(function(event) {
    return event.name === "App quit";
  })
  .map(function(item) {
    return {
      'length': item.properties['minutes since start']
    };
  });
}`;


var sessionsPerUser = `function main() {
  function getDay(event) {
    return (new Date(event.time)).toISOString().split('T')[0];
  }
  return Events({
    from_date: '${helpers.formatDate(monthAgo)}',
    to_date: '${helpers.formatDate(today)}'
  })
  .filter(function(event) {
    return event.name === "App quit";
  })
  .filter(function(event) {
    return /^((?!(dev|world|testing)).)*$/.test(event.properties['app version']);
  })
  .map(function(event) {
    var obj = {
      id: event.distinct_id,
      length: event.properties['minutes since start'],
      date: getDay(event)
    };
    return obj;
  })
  .reduce(function(accumulators, items) {
    var i;
    var obj = {};
    for(i = 0; i < accumulators.length; ++i) {
      for (var d in accumulators[i]) {
        if (obj[d] === undefined) {
          obj[d] = {};
        }
        for (var u in accumulators[i][d]) {
          if (obj[d][u] === undefined) {
            obj[d][u] = {
              sessions: accumulators[i][d][u].sessions,
              lengths: accumulators[i][d][u].lengths
            };
          }
          else {
            obj[d][u].sessions += accumulators[i][d][u].sessions;
            obj[d][u].lengths += accumulators[i][d][u].lengths;
          }
        }
      }
    }
    for(i = 0; i < items.length; ++i) {
      var date = items[i].date;
      var id = items[i].id;
      var length = items[i].length;
      if (obj[date] === undefined) {
        obj[date] = {};
      }
      if (obj[date][id] === undefined) {
        obj[date][id] = {
          sessions: 1,
          lengths: length
        };
      }
      else {
        obj[date][id].sessions += 1;
        obj[date][id].sessions += length;
      }
    }
    return obj;
  });
}`;


function getSessionStats(secret) {
  var str = mixpanel.mixpanelJQL(secret, sessionStatsQuery);
  return str;
}

function getSessionsPerUser(secret) {
  var str = mixpanel.mixpanelJQL(secret, sessionsPerUser);
  var str2 = helpers.transformStreamJQL(str);
  return str2;
}

module.exports = {
  getSessionsPerUser: getSessionsPerUser,
  getSessionStats: getSessionStats
};
