require('dotenv').config();
var intercom = require('mongodb-js-intercom');

// source: https://en.wikipedia.org/wiki/List_of_Microsoft_Windows_versions
var windowsObj = {
  '6.0;': 'Windows Vista',
  '6.1;': 'Windows 7',
  '6.2;': 'Windows 8',
  '6.3;': 'Windows 8.1',
  '10.0;': 'Windows 10'
};

// source https://en.wikipedia.org/wiki/OS_X
var macObj = {
  '10_12_0': 'macOs Sierra',
  '10_11_6': 'Mac OS X El Capitan',
  '10_11_5': 'Mac OS X El Capitan',
  '10_11_4': 'Mac OS X El Capitan',
  '10_11_3': 'Mac OS X El Capitan',
  '10_11_2': 'Mac OS X El Capitan',
  '10_11_1': 'Mac OS X El Capitan',
  '10_11_0': 'Mac OS X El Capitan',
  '10_10_5': 'Mac OS X Yosemite',
  '10_10_4': 'Mac OS X Yosemite',
  '10_10_3': 'Mac OS X Yosemite',
  '10_10_2': 'Mac OS X Yosemite',
  '10_10_1': 'Mac OS X Yosemite',
  '10_10_0': 'Mac OS X Yosemite',
  '10_9_5': 'Mac OS X Maverick',
  '10_9_4': 'Mac OS X Maverick',
  '10_9_3': 'Mac OS X Maverick',
  '10_9_2': 'Mac OS X Maverick',
  '10_9_1': 'Mac OS X Maverick',
  '10_9': 'Mac OS X Maverick',
  '10_8_5': 'Mac OS X Mountain Lion',
  '10_8_4': 'Mac OS X Mountain Lion',
  '10_8_3': 'Mac OS X Mountain Lion',
  '10_8_2': 'Mac OS X Mountain Lion',
  '10_8_1': 'Mac OS X Mountain Lion',
  '10_8': 'Mac OS X Mountain Lion'
};

var regexParen = /\([^\)]+\)/;
var regexMac = /(?:\S+ ){5}([^(\s|\))]+)/;
var regexWindows = /(?:\S+ ){2}([^(\s|\))]+)/;

/*
**   Function that uses the mongodb-js-intercom to fetch
** all users in the Active segment created on the Intercom
** website. The code then iterates through all users, using
** RegEx to parse through their data and pull out the
** operating system type
*/
var getActiveUsers = function() {
  var osDict = {
    'Total': 0,
    'Mac': 0,
    'macOs Sierra': 0,
    'Mac OS X El Capitan': 0,
    'Mac OS X Yosemite': 0,
    'Mac OS X Maverick': 0,
    'Mac OS X Mountain Lion': 0,
    'Windows': 0,
    'Windows 10': 0,
    'Windows 8.1': 0,
    'Windows 8': 0,
    'Windows 7': 0,
    'Linux': 0,
    'undefined': 0
  };

  // creates a promise that resolves with a dictionary
  // containg key value pairs of the active user breakdown by OS
  var p = new Promise(function(resolve, reject) {
    intercom.getUsersBySegment('Active', function(error, results) {
      if (error) {
        return reject(error);
      }
      results.on('data', function(chunk) {
        if (chunk.custom_attributes.app_stage === 'development' ||
              chunk.custom_attributes.app_stage === 'testing') {
          return;
        }
        var info;
        try {
          info = regexParen.exec(chunk.user_agent_data)[0];
        } catch (err) {
          osDict.undefined++;
          return;
        }
        var os;
        if (/Mac/.exec(info)) {
          osDict.Mac++;
          os = regexMac.exec(info)[1];
          if (osDict[macObj[os]]) {
            osDict[macObj[os]]++;
          } else {
            osDict[macObj[os]] = 1;
          }
        } else if (/Win/.exec(info)) {
          osDict.Windows++;
          os = regexWindows.exec(info)[1];
          if (osDict[windowsObj[os]]) {
            osDict[windowsObj[os]]++;
          } else {
            osDict[windowsObj[os]] = 1;
          }
        } else if (/X11/.exec(info)) {
          osDict.Linux++;
        } else {
          osDict.undefined++;
        }
        osDict.Total++;
      });
      results.on('end', function() {
        resolve(osDict);
      });
    });
  });
  return p;
};

module.exports = getActiveUsers;
